@@ -2,6 +2,9 @@
  * here we list all symbols, which get introduced by configure.in
  */
 
+#include "PKG_paths.h"
+
+@TOP@
 
 #undef  BST_VERSION
 #undef  BST_DFL_KEYTABLE
@@ -9,3 +12,5 @@
 #undef  BST_WITH_GDK_PIXBUF
 #undef  BSE_PCM_DEVICE_CONF_ALSA
 #undef  BSE_PCM_DEVICE_CONF_OSS
+
+@BOTTOM@
