@@ -73,7 +73,15 @@ typedef enum
   /* application wide
    */
   BST_OP_EXIT,
+
+  /* help dialogs
+   */
+#define BST_OP_HELP_FIRST	BST_OP_HELP_ABOUT
   BST_OP_HELP_ABOUT,
+  BST_OP_HELP_FAQ,
+  BST_OP_HELP_HEART,
+  BST_OP_HELP_NETWORKS,
+#define	BST_OP_HELP_LAST	BST_OP_HELP_NETWORKS
 
   BST_OP_LAST
 } BstOps;
@@ -99,7 +107,7 @@ BseIcon* bst_icon_from_stock (BstIconId icon_id);
 
 
 /* --- debug stuff --- */
-typedef enum                    /* <skip> */
+typedef enum                    /*< skip >*/
 { /* keep in sync with bstmain.c */
   BST_DEBUG_KEYTABLE		= (1 << 0),
   BST_DEBUG_SAMPLES		= (1 << 1)
@@ -112,6 +120,8 @@ extern BstDebugFlags bst_debug_flags;
 #endif /* !G_ENABLE_DEBUG */
 
 
+#define BST_STRDUP_RC_FILE()	(g_strconcat (g_get_home_dir (), "/.beastrc", NULL))
+
 /* it's hackish to have these prototypes in here, but we need
  * 'em somewhere, implementations are in bstmain.c
  */
@@ -126,9 +136,12 @@ extern void bst_object_set         (gpointer     object,
 
 GtkWidget* bst_subwindow_new       (GtkObject	 *alive_host,
 				    GtkWidget	**ssubwindow_p,
-				    GtkWidget	 *child);
+				    GtkWidget	 *child,
+				    guint	  zero);
 GtkWidget* bst_subwindow_get_child (GtkWidget	 *subwindow);
 void	   gtk_widget_showraise    (GtkWidget	 *widget);
+void	   gtk_toplevel_hide       (GtkWidget    *widget);
+
 
 /* --- canvas utils/workarounds --- */
 extern GnomeCanvasPoints* gnome_canvas_points_new0 (guint num_points);
