@@ -17,6 +17,7 @@
  */
 #include "bstpreferences.h"
 
+#include "bstgconfig.h"
 
 
 /* --- prototypes --- */
@@ -75,10 +76,35 @@ bst_preferences_init (BstPreferences *prefs)
   GtkWidget *any, *hbox, *button;
 
   prefs->gconf = NULL;
-  prefs->param_box = gtk_widget_new (BST_TYPE_PARAM_VIEW, NULL);
-  BST_PARAM_VIEW (prefs->param_box)->base_type = BSE_TYPE_GCONFIG;
-  gtk_widget_show (prefs->param_box);
-  gtk_container_add (GTK_CONTAINER (prefs), prefs->param_box);
+  prefs->bse_param_view = gtk_widget_new (BST_TYPE_PARAM_VIEW,
+					  "visible", TRUE,
+					  "object_signal::destroy", bse_nullify_pointer, &prefs->bse_param_view,
+					  NULL);
+  bst_param_view_set_mask (BST_PARAM_VIEW (prefs->bse_param_view), BSE_TYPE_GCONFIG, BSE_TYPE_GCONFIG, NULL, NULL);
+  prefs->bst_param_view = gtk_widget_new (BST_TYPE_PARAM_VIEW,
+					  "object_signal::destroy", bse_nullify_pointer, &prefs->bst_param_view,
+					  NULL);
+  bst_param_view_set_mask (BST_PARAM_VIEW (prefs->bst_param_view), BST_TYPE_GCONFIG, 0, NULL, NULL);
+
+  prefs->notebook = gtk_widget_new (GTK_TYPE_NOTEBOOK,
+				    "visible", TRUE,
+				    "parent", prefs,
+				    "tab_pos", GTK_POS_TOP,
+				    "scrollable", FALSE,
+				    "can_focus", TRUE,
+				    "border_width", 5,
+				    "object_signal::destroy", bse_nullify_pointer, &prefs->notebook,
+				    NULL);
+  gtk_notebook_append_page (GTK_NOTEBOOK (prefs->notebook), prefs->bst_param_view,
+			    gtk_widget_new (GTK_TYPE_LABEL,
+					    "visible", TRUE,
+					    "label", "BEAST",
+					    NULL));
+  gtk_notebook_append_page (GTK_NOTEBOOK (prefs->notebook), prefs->bse_param_view,
+			    gtk_widget_new (GTK_TYPE_LABEL,
+					    "visible", TRUE,
+					    "label", "BSE",
+					    NULL));
 
   /* dialog bits
    */
@@ -99,27 +125,15 @@ bst_preferences_init (BstPreferences *prefs)
 			   "visible", TRUE,
 			   "can_default", TRUE,
 			   "object_signal::clicked", bst_preferences_apply, prefs,
+			   "object_signal::clicked", bst_preferences_save, prefs,
 			   "object_signal::destroy", bse_nullify_pointer, &prefs->apply,
 			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->param_box)->tooltips, button,
-			"Apply the preference values. Some values may only take effect after"
-			"a restart. The preference values are locked against modifcation during"
+  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->bse_param_view)->tooltips, button,
+			"Apply and save the preference values. Some values may only take effect after "
+			"restart. The preference values are locked against modifcation during "
 			"playback.",
 			NULL);
   prefs->apply = button;
-  button = gtk_widget_new (GTK_TYPE_BUTTON,
-			   "label", "Save",
-			   "parent", hbox,
-			   "visible", TRUE,
-			   "can_default", TRUE,
-			   "sensitive", FALSE,
-			   //"object_signal::clicked", bst_preferences_apply, prefs,
-			   "object_signal::destroy", bse_nullify_pointer, &prefs->save,
-			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->param_box)->tooltips, button,
-			"Save the preference values (does not apply them to the running program).",
-			NULL);
-  prefs->save = button;
   button = gtk_widget_new (GTK_TYPE_BUTTON,
 			   "label", "Revert",
 			   "parent", hbox,
@@ -127,7 +141,7 @@ bst_preferences_init (BstPreferences *prefs)
 			   "can_default", TRUE,
 			   "object_signal::clicked", bst_preferences_revert, prefs,
 			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->param_box)->tooltips, button,
+  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->bse_param_view)->tooltips, button,
 			"Revert the preference values to the current internal values.",
 			NULL);
   button = gtk_widget_new (GTK_TYPE_BUTTON,
@@ -137,9 +151,18 @@ bst_preferences_init (BstPreferences *prefs)
 			   "can_default", TRUE,
 			   "object_signal::clicked", bst_preferences_default_revert, prefs,
 			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->param_box)->tooltips, button,
-			"Revert to hardcoded default values (factory settings ;).",
+  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->bse_param_view)->tooltips, button,
+			"Revert to hardcoded default values (factory settings).",
 			NULL);
+  button = gtk_widget_new (GTK_TYPE_BUTTON,
+			   "label", "Close",
+			   "parent", hbox,
+			   "visible", TRUE,
+			   "can_default", TRUE,
+			   "signal::clicked", gtk_toplevel_hide, NULL,
+			   "object_signal::destroy", bse_nullify_pointer, &prefs->close,
+			   NULL);
+  prefs->close = button;
 }
 
 static void
@@ -181,7 +204,8 @@ bst_preferences_set_gconfig (BstPreferences *prefs,
 
   if (prefs->gconf)
     {
-      bst_param_view_set_object (BST_PARAM_VIEW (prefs->param_box), NULL);
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bse_param_view), NULL);
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bst_param_view), NULL);
       bse_object_remove_notifiers_by_func (prefs->gconf,
 					   preferences_lock_changed,
 					   prefs);
@@ -196,7 +220,15 @@ bst_preferences_set_gconfig (BstPreferences *prefs,
 				    "lock_changed",
 				    preferences_lock_changed,
 				    prefs);
-      bst_param_view_set_object (BST_PARAM_VIEW (prefs->param_box), BSE_OBJECT (prefs->gconf));
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bse_param_view), BSE_OBJECT (prefs->gconf));
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bst_param_view), BSE_OBJECT (prefs->gconf));
+      if (bse_type_next_base (BSE_OBJECT_TYPE (prefs->gconf), BSE_TYPE_GCONFIG))
+	{
+	  gtk_widget_show (prefs->bst_param_view);
+	  gtk_notebook_set_page (GTK_NOTEBOOK (prefs->notebook), 0);
+	}
+      else
+	gtk_widget_hide (prefs->bst_param_view);
     }
   preferences_lock_changed (prefs);
 }
@@ -226,6 +258,21 @@ bst_preferences_apply (BstPreferences *prefs)
   bse_gconfig_apply (prefs->gconf);
 }
 
+void
+bst_preferences_save (BstPreferences *prefs)
+{
+  BseErrorType error;
+  gchar *file_name;
+
+  g_return_if_fail (BST_IS_PREFERENCES (prefs));
+
+  file_name = BST_STRDUP_RC_FILE ();
+  error = bst_rc_dump (file_name, prefs->gconf);
+  if (error)
+    g_warning ("error saving rc-file \"%s\": %s", file_name, bse_error_blurb (error));
+  g_free (file_name);
+}
+
 void
 bst_preferences_revert (BstPreferences *prefs)
 {
@@ -261,13 +308,14 @@ ifactory_print_func (gpointer  user_data,
 }
 
 BseErrorType
-bst_rc_dump (const gchar *file_name)
+bst_rc_dump (const gchar *file_name,
+	     BseGConfig  *gconf)
 {
   BseStorage *storage;
-  BseGConfig *gconf;
   gint fd;
 
   g_return_val_if_fail (file_name != NULL, BSE_ERROR_INTERNAL);
+  g_return_val_if_fail (BSE_IS_GCONFIG (gconf), BSE_ERROR_INTERNAL);
 
   fd = open (file_name,
 	     O_WRONLY | O_CREAT | O_TRUNC, /* O_EXCL, */
@@ -289,14 +337,11 @@ bst_rc_dump (const gchar *file_name)
    */
   bse_storage_break (storage);
   bse_storage_puts (storage, "; BseGlobals dump");
-  gconf = bse_object_new (BSE_TYPE_GCONFIG, NULL);
-  bse_gconfig_revert (gconf);
   bse_storage_break (storage);
   bse_storage_puts (storage, "(preferences");
   bse_storage_push_level (storage);
   bse_object_store (BSE_OBJECT (gconf), storage);
   bse_storage_pop_level (storage);
-  bse_object_unref (BSE_OBJECT (gconf));
 
   bse_storage_break (storage);
 
@@ -321,7 +366,8 @@ bst_rc_dump (const gchar *file_name)
 }
 
 BseErrorType
-bst_rc_parse (const gchar *file_name)
+bst_rc_parse (const gchar *file_name,
+	      BseGConfig  *gconf)
 {
   BseStorage *storage;
   GScanner *scanner;
@@ -329,7 +375,8 @@ bst_rc_parse (const gchar *file_name)
   BseErrorType error;
 
   g_return_val_if_fail (file_name != NULL, BSE_ERROR_INTERNAL);
-
+  g_return_val_if_fail (BSE_IS_GCONFIG (gconf), BSE_ERROR_INTERNAL);
+  
   storage = bse_storage_new ();
   error = bse_storage_input_file (storage, file_name);
   if (error)
@@ -349,15 +396,7 @@ bst_rc_parse (const gchar *file_name)
 	{
 	  g_scanner_get_next_token (scanner);
 	  if (bse_string_equals ("preferences", scanner->value.v_identifier))
-	    {
-	      BseGConfig *gconf = bse_object_new (BSE_TYPE_GCONFIG, NULL);
-
-	      bse_gconfig_revert (gconf);
-	      expected_token = bse_object_restore (BSE_OBJECT (gconf), storage);
-	      if (expected_token == G_TOKEN_NONE)
-		bse_gconfig_apply (gconf);
-	      bse_object_unref (BSE_OBJECT (gconf));
-	    }
+	    expected_token = bse_object_restore (BSE_OBJECT (gconf), storage);
 	  else if (bse_string_equals ("menu-accelerators", scanner->value.v_identifier))
 	    {
 	      guint symbol_2_token = scanner->config->symbol_2_token;
