@@ -33,6 +33,7 @@
 #include        <bse/bseconfig.h>
 #include        <bse/bsecontainer.h>
 #include        <bse/bsedefs.h>
+#include        <bse/bsedevice.h>
 #include        <bse/bseeffect.h>
 #include        <bse/bseeffectfinetune.h>
 #include        <bse/bseeffectnotevolume.h>
@@ -41,6 +42,7 @@
 #include        <bse/bseeffectvolumedelta.h>
 #include        <bse/bseenums.h>
 #include        <bse/bseexports.h>
+#include        <bse/bsegconfig.h>
 #include        <bse/bseglobals.h>
 #include        <bse/bseheart.h>
 #include        <bse/bseinstrument.h>
