@@ -80,14 +80,14 @@ typedef enum
   BSE_LOOP_PATTERN,
   BSE_LOOP_PATTERN_ROWS,
   BSE_LOOP_SONG,
-  BSE_LOOP_LAST				/* <skip> */
+  BSE_LOOP_LAST				/*< skip >*/
 } BseLoopType;
 typedef enum
 {
   BSE_INSTRUMENT_NONE,
   BSE_INSTRUMENT_SAMPLE,
   BSE_INSTRUMENT_MIDI,
-  BSE_INSTRUMENT_LAST			/* <skip> */
+  BSE_INSTRUMENT_LAST			/*< skip >*/
 } BseInstrumentType;
 typedef enum
 {
@@ -96,8 +96,10 @@ typedef enum
   BSE_ERROR_UNKNOWN,
   BSE_ERROR_INTERNAL,
   BSE_ERROR_UNIMPLEMENTED,
+  BSE_ERROR_IO,
+  BSE_ERROR_PERMS,
   /* FIle, Loading/Saving errors */
-  BSE_ERROR_FILE_IO,
+#define BSE_ERROR_FILE_IO	BSE_ERROR_IO
   BSE_ERROR_FILE_EXISTS,
   BSE_ERROR_FILE_NOT_FOUND,
   BSE_ERROR_FILE_TOO_SHORT,
@@ -111,10 +113,10 @@ typedef enum
   BSE_ERROR_BINARY_DATA_CORRUPT,
   BSE_ERROR_PARSE_ERROR,
   /* Device errors */
-  BSE_ERROR_DEVICE_PERMS,
+#define BSE_ERROR_DEVICE_IO	BSE_ERROR_IO
+#define BSE_ERROR_DEVICE_PERMS	BSE_ERROR_PERMS
   BSE_ERROR_DEVICE_ASYNC,
   BSE_ERROR_DEVICE_BUSY,
-  BSE_ERROR_DEVICE_IO,
   BSE_ERROR_DEVICE_GET_CAPS,
   BSE_ERROR_DEVICE_CAPS_MISMATCH,
   BSE_ERROR_DEVICE_SET_CAPS,
@@ -141,7 +143,7 @@ typedef enum
   BSE_ERROR_PROC_EXECUTION,
   BSE_ERROR_PROC_NEEDLESS,
   BSE_ERROR_PROC_ABORT,
-  BSE_ERROR_LAST			/* <skip> */
+  BSE_ERROR_LAST			/*< skip >*/
 } BseErrorType;
 
 
@@ -164,8 +166,16 @@ BseFlagsValue*	bse_flags_get_value_by_nick	(BseFlagsClass	*flags_class,
 gchar*		bse_error_name			(BseErrorType	 error_value);
 gchar*		bse_error_nick			(BseErrorType	 error_value);
 gchar*		bse_error_blurb			(BseErrorType	 error_value);
+BseErrorType	bse_error_from_errno		(gint		 v_errno,
+						 BseErrorType    fallback);
 
 
+/* --- implementation details --- */
+const gchar*    bse_enum_type_register          (const gchar *name,
+						 BseType      parent_type,
+						 BsePlugin   *plugin,
+						 BseType     *ret_type);
+
 
 #ifdef __cplusplus
 }
