@@ -351,7 +351,7 @@ static void
 bse_object_do_set_name (BseObject   *object,
 			const gchar *name)
 {
-  g_datalist_id_set_data_full (&object->datalist, _bse_quark_name, g_strdup (name), g_free);
+  g_datalist_id_set_data_full (&object->datalist, _bse_quark_name, g_strdup (name), name ? g_free : NULL);
 }
 
 static void
@@ -1168,7 +1168,7 @@ bse_object_class_add_param (BseObjectClass *class,
     {
       if (class->param_specs[i]->any.param_id == param_id)
 	{
-	  g_warning ("bse_object_class_add_param(): class `%s' already contains a parameter for %u",
+	  g_warning ("bse_object_class_add_param(): class `%s' already contains a parameter with id %u",
 		     BSE_CLASS_NAME (class),
 		     param_id);
 	  return;
@@ -1611,7 +1611,14 @@ bse_object_do_restore_private (BseObject  *object,
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER)
     return BSE_TOKEN_UNMATCHED;
   
-  /* ok, we got an identifier, try object parameter lookup */
+  /* ok, we got an identifier, try object parameter lookup
+   * we should in theory only get BSE_PARAM_SERVE_STORAGE
+   * parameters here, but due to version changes or even
+   * users editing their files, we will simply parse all
+   * kinds of parameters here (we might want to at least
+   * restrict them to BSE_PARAM_SERVE_STORAGE and
+   * BSE_PARAM_SERVE_GUI at some point...)
+   */
   pspec = bse_object_class_get_param_spec (BSE_OBJECT_GET_CLASS (object),
 					   scanner->next_value.v_identifier);
   if (!pspec)
